#!/bin/bash

if test `id -u` -ne 0; then
	echo "Must be root to run this script"
	exit 1;
fi

uudecode="/usr/bin/uudecode"
openvpn="/usr/sbin/openvpn"

export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:$PATH

for bin in uudecode openvpn; do
    if test "`which $bin`" == ""; then
        echo "you'll need '$bin' to run this script"
        echo -e "\tuudecode is in the 'sharutils' package\n\topenvpn is in the 'openvpn' package\n"
        echo -e -n "\n\nDo you want to install them (Y/n): "
        read yn
        if [ "$yn" != "n" ]; then
          yum install -y sharutils openvpn || exit 1
        fi
    fi
done


uudecode tmp/openvpn.uu 
d=`date +%F`
echo "Backing up current openvpn directory"
tar czf /etc/openvpn.$d.tar.gz /etc/openvpn

echo "Untar'ing install files"
tar xzf openvpn.taz -C /etc

[ ! -d /var/log/openvpn ] && mkdir -p /var/log/openvpn
[ ! -e /etc/openvpn/logs ] && ln -s /var/log/openvpn /etc/openvpn/logs

chmod 400 /etc/openvpn/<SERVER>/ta.key /etc/openvpn/<SERVER>/ca.* /etc/openvpn/<SERVER>/dh*.pem
echo "Cleanup"
rm tmp/openvpn.uu openvpn.taz tmp/install.sh

chmod +x /etc/openvpn/<SERVER>/*.sh
chmod 644 /etc/openvpn/<SERVER>/crl.pem
[ ! -d /etc/openvpn/gateway ] && mkdir /etc/openvpn/gateway

chmod +x /etc/openvpn/taptunadjust.sh
/etc/openvpn/taptunadjust.sh
rm /etc/openvpn/taptunadjust.sh

echo "Restarting..."

[ -f /etc/init.d/openvpn ] && /etc/init.d/openvpn restart

echo "Done"

rm -rf /tmp/ovpninstall
