#!/bin/bash

. /etc/openvpncertmgmt.conf


#if test `id -u` -ne 0; then
#	echo "Must be root to run this script"
#	exit 1;
#fi

CLIENT=${1}
LOCATION=${2}
SERVERDNS=${3}
PORT=${4}

USERNAME=${5}
PASSWORD=${6}
AUTHENTICATIONMETHOD=${7}
HTTPPROXYHOST=${8}
HTTPPROXYRETRY=${9}
HTTPPROXYTIMEOUT=${10}
HTTPPROXYOPTION_VERSION=${10}
HTTPPROXYOPTION_AGENT=${12}
VERB=${13}

NAME="${1}-${2}"

if test -f $OPENVPNHOME/easy-rsa/keys/${CLIENT}/$NAME.crt; then
	echo -n "CLIENT crt already exists!  Exiting... "
	exit 1
fi

cd $OPENVPNHOME/easy-rsa
mkdir $OPENVPNHOME/openvpn
rm -fr $OPENVPNHOME/openvpn/*
mkdir $OPENVPNHOME/openvpn/$NAME

#if test "$use_existing" != "true"; then
    . $OPENVPNHOME/easy-rsa/keys/${CLIENT}/vars
    export KEY_DIR="$EASY_RSA/keys/${CLIENT}"

    $OPENVPNHOME/easy-rsa/build-key --batch $NAME
    cp $KEY_DIR/$NAME.crt $KEY_DIR/$NAME.crt.pre
    echo -e "y\ny\n" | $OPENVPNHOME/easy-rsa/sign-req --batch $NAME 2>&1
    
    cd $KEY_DIR
    $OPENSSL ca -revoke "$NAME.crt.pre" -config "$KEY_CONFIG" 2>&1
    cd $OPENVPNHOME/easy-rsa

#fi

cd $OPENVPNHOME/clients
cp $OPENVPNHOME/scripts/client.conf.in $OPENVPNHOME/clients/${NAME}.conf

sed -i "s/<SERVERDNS>/${SERVERDNS}/g" $OPENVPNHOME/clients/${NAME}.conf
sed -i "s/<PORT>/${PORT}/g" $OPENVPNHOME/clients/${NAME}.conf
sed -i "s/<CLIENT>/${NAME}/g" $OPENVPNHOME/clients/${NAME}.conf

sed -i "s/<PROTOCOL>/${PROTOCOL}/g" $OPENVPNHOME/clients/${NAME}.conf
sed -i "s/<TUNTAP>/${TUNTAP}/g" $OPENVPNHOME/clients/${NAME}.conf

#
# Advanced Options
#
sed -i "s/<VERB>/$VERB/" $OPENVPNHOME/clients/${NAME}.conf

if [ "$HTTPPROXYHOST" != "" ]; then
    sed -i "s/#<HTTP-PROXY>/http-proxy/" $OPENVPNHOME/clients/${NAME}.conf
    sed -i "s/<HTTPPROXYHOST>/$HTTPPROXYHOST/" $OPENVPNHOME/clients/${NAME}.conf
    if [ "$USERNAME" != "" ]; then
        sed -i "s/<AUTHFILE>/${NAME}\/authfile/" $OPENVPNHOME/clients/${NAME}.conf
        sed -i "s/<AUTH-METHOD>/$AUTHENTICATIONMETHOD/" $OPENVPNHOME/clients/${NAME}.conf
        echo ${USERNAME} >$OPENVPNHOME/openvpn/${NAME}/authfile
        echo ${PASSWORD} >>$OPENVPNHOME/openvpn/${NAME}/authfile
    else
        sed -i "s/<AUTHFILE>//" $OPENVPNHOME/clients/${NAME}.conf
        sed -i "s/<AUTH-METHOD>//" $OPENVPNHOME/clients/${NAME}.conf
    fi

    [ "$HTTPPROXYRETRY" = "yes" ] && sed -i "s/#<HTTP-PROXY-RETRY>/http-proxy-retry/" $OPENVPNHOME/clients/${NAME}.conf
    sed -i "s/#<HTTP-PROXY-TIMEOUT>/http-proxy-timeout $HTTPPROXYTIMEOUT/" $OPENVPNHOME/clients/${NAME}.conf
    [ "$HTTPPROXYOPTION_VERSION" != "" ]  && sed -i "s/#<HTTP-PROXY-OPTION-VERSION>/http-proxy-option VERSION \"$HTTPPROXYOPTION_VERSION\"/" $OPENVPNHOME/clients/${NAME}.conf
    [ "$HTTPPROXYOPTION_AGENT" != "" ] && sed -i "s/#<HTTP-PROXY-OPTION-AGENT>/http-proxy-option AGENT \"$HTTPPROXYOPTION_AGENT\"/" $OPENVPNHOME/clients/${NAME}.conf
fi


if [ "$OPENVPNVERSION_21" = "yes" ]; then
    sed -i "s/<2.1>//g" $OPENVPNHOME/clients/${SERVER}.conf
else
    sed -i "s/<2.1>/# Openvpn 2.1 and above: /g" $OPENVPNHOME/clients/${NAME}.conf
fi


#
# Set up for Macs
#
cp $OPENVPNHOME/clients/${NAME}.conf $OPENVPNHOME/clients/${NAME}.mac.conf
sed -i "s/<#>/#/g" $OPENVPNHOME/clients/${NAME}.mac.conf


#
# Set up for Windows
#
cp $OPENVPNHOME/clients/${NAME}.conf $OPENVPNHOME/clients/${NAME}.ovpn

/usr/bin/unix2dos $OPENVPNHOME/clients/${NAME}.ovpn

sed -i "s/<#>//g" $OPENVPNHOME/clients/${NAME}.conf
sed -i "s/<#>/#/g" $OPENVPNHOME/clients/${NAME}.ovpn


for i in ca.crt $NAME.crt $NAME.key ta.key; do
    cp $OPENVPNHOME/easy-rsa/keys/$CLIENT/$i $OPENVPNHOME/openvpn/${NAME}
done

cd $OPENVPNHOME

#
# The install.sh is used for all Unixes
#

cp scripts/install.sh $OPENVPNHOME/tmp/
sed -i "s/<SERVER>/${NAME}/g" $OPENVPNHOME/tmp/install.sh


set -x

#
# Create the Windows zip file
#
cp $OPENVPNHOME/clients/$NAME.ovpn $OPENVPNHOME/openvpn
cd $OPENVPNHOME/openvpn && zip -9r $OPENVPNHOME/clients/installscripts/openvpn-${NAME}.zip *
rm  $OPENVPNHOME/openvpn/$NAME.ovpn

#####################################
#
# Following is all for Linux, Unix & Mac
#
#####################################


#
# Shell scripts
#
cp $OPENVPNHOME/scripts/up.sh $OPENVPNHOME/openvpn
cp $OPENVPNHOME/scripts/down.sh $OPENVPNHOME/openvpn

#
# Taptunadjust.  Copied to /etc/openvpn for use during install, then
# deleted.
#
dev=`expr substr $TUNTAP 1 3`
sed 's/<DEV>/'"$dev"'/' $OPENVPNHOME/scripts/taptunadjust.sh |
    sed 's/<CONF>/'"$NAME"'/g' >$OPENVPNHOME/openvpn/taptunadjust.sh


#
# Create install file for Linux/Unix
#
cp $OPENVPNHOME/clients/$NAME.conf $OPENVPNHOME/openvpn
cd $OPENVPNHOME && tar cz openvpn | uuencode openvpn.taz > $OPENVPNHOME/tmp/openvpn.uu
cd $OPENVPNHOME && shar tmp/install.sh tmp/openvpn.uu >$OPENVPNHOME/clients/installscripts/openvpn-${NAME}.sh
sed -i 's/exit 0/.\/tmp\/install.sh\nexit 0/g' $OPENVPNHOME/clients/installscripts/openvpn-${NAME}.sh

# Now for Macs
rm $OPENVPNHOME/openvpn/$NAME.conf
cp $OPENVPNHOME/clients/$NAME.mac.conf $OPENVPNHOME/openvpn
cd $OPENVPNHOME && tar cz openvpn | uuencode openvpn.taz > $OPENVPNHOME/tmp/openvpn.uu
cd $OPENVPNHOME && shar tmp/install.sh tmp/openvpn.uu >$OPENVPNHOME/clients/installscripts/openvpn-${NAME}.mac.sh

rm $OPENVPNHOME/tmp/install.sh


rm -f $OPENVPNHOME/clients/installscripts/openvpn-${NAME}*.bak $OPENVPNHOME/clients/${NAME}*.bak 
rm -fr $OPENVPNHOME/openvpn/tmp/*

exit   
