#!/bin/bash

CONFIG_FILES="www/status/plugins/email/config.inc www/status/plugins/revokecrt/local.inc www/status/plugins/win32installer/local.inc www/status/config.inc"

if test `id -u` -ne 0; then
       echo "Must be root to run this script"
       exit 1;
fi

if [ "$1" = "save" ]; then
    tar czf openvpn.config.tar.gz $CONFIG_FILES servers clients easy-rsa/keys
    exit 0;
fi    

if [ "$1" = "distclean" ]; then
    echo -n "Are you sure you want to delete all configuration, client and server files? "
    read yn
    if [ "$yn" = "y" ]; then
        echo "Deleting..."
        for i in $CONFIG_FILES; do
            echo $i
            rm -f $i
        done
        
        rm -fr servers/installscripts/*
        rm -f servers/*.conf servers/*.ovpn servers/*.bak
        rm -fr clients/installscripts/*
        rm -f clients/*.conf clients/*.ovpn clients/*.bak
        rm -fr easy-rsa/keys/*
        
        rm -fr www/status/templates_c/*
        rm -fr www/status/Smarty-2.6.26
        
        rm -f clients/downloadRequests.txt
        rm -f windows/* www/status/plugins/win32installer/tmp/*
        
        find . -name .svn -exec rm -fr {} \; 
        find . -name \*~ -exec rm {} \; 
    fi

    exit 0
fi

if [ "$1" = "dist" ]; then
    cd /home
    if [ $# != 2 ]; then
        echo "Must specify a destination tar.gz file"
        exit 1
    fi
    a=$2
    lchr=`expr substr $a ${#a} 1`
    if [ "$lchr" != "z" ]; then
        echo "Must be a tar.gz file"
        exit 1
    fi
    tar czf $2 openvpn
    exit 0
fi


tolower()
{
    local char="$*"
    
    out=$(echo $char | tr [:upper:] [:lower:])
    local retval=$?
    echo "$out"
    unset out
    unset char
    return $retval
}

echo -e "\n\n\n\n\n"

echo -n "Are you sure you want to install the default config files? "
read yn

yn2=$(tolower $yn)
if [ "$yn2" = "y" ]; then
    echo -n "Are you sure (this will overwrite any existing config files)? "
    read yn
    yn2=$(tolower $yn)
    if [ "$yn2" = "y" ]; then
        for i in $CONFIG_FILES; do
            [ -f $i ] && mv $i $i.old
            cp $i.in $i
            echo $i
        done
        cp openvpncertmgmt.conf /etc
    fi

fi

touch clients/downloadRequests.txt


echo -n "Enter the Apache user: "
read user

chown -R $user.$user . *

