<?php

/**
 * Project:     SmartyValidate: Form Validator for the Smarty Template Engine
 * File:        validate_criteria.isDNSAddress.php
 * Author:      Jonathan Bayer (Regiscope)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/**
 * test if a value is a valid DNS address or IP address
 *
 * @param string $value the value being tested
 * @param boolean $empty if field can be empty
 * @param array params validate parameter values
 * @param array formvars form var values
 */
function smarty_validate_criteria_isDNSAddress($value, $empty, &$params, &$formvars) 
{
  if (strlen($value) == 0 && $empty) return true;
//  if (gethostbyname($value) != $value) return true;
  if (getAddrByHost($value) != $value) return true;
  if (strlen($value) == 0) return false;
  
  if(isset($params['field2'])) 
  {
    if ($params['field2'] == "allowUnresolvable") return true;
  }
  return false;
}


function getAddrByHost($host, $timeout = 3) 
{
  $dig = `which dig`;
  if (strlen($dig) > 0)
  {
    //
    // First use the local DNS, then go external
    //
    $query = `dig +time=$timeout +retry=1 -t A +short $host`;
    if (strlen($query) > 2) return $query;
    $query = `dig @4.2.2.2 +time=$timeout +retry=1 -t A +short $host`;
    if (strlen($query) > 2) return $query;
    return $host;    
  }
  
  $nslookup = `which nslookup`;
  if (strlen($nslookup) > 0)
  {
    //
    // First use the local DNS, then go external
    //
    $query = `nslookup -timeout=$timeout -retry=1 $host`;
    if(preg_match('/\nAddress: (.*)\n/', $query, $matches)) 
      return trim($matches[1]);
    $query = `nslookup -timeout=$timeout -retry=1 $host 4.2.2.2`;
    if(preg_match('/\nAddress: (.*)\n/', $query, $matches)) 
      return trim($matches[1]);
    return $host;
  }
  // No lookup possible, just return localhost
  return "127.0.0.1";

}
 
?>
