<?
//
// Adapted from: http://forums.dotproject.net/showthread.php?t=8208
//

if (!defined ("LOCAL_SECURITY_KEY"))
 die ();

 
// To expand this with new authentication methods, create a new class
// which is extends the Authenticator class, and write the necessary
// authentication functions for it, then add the new method to the
// getAuth() function.
 
function &getAuth($auth_mode)
{
  switch($auth_mode)
  {
    case "ldap":
      $auth = new LDAPAuthenticator();
      $auth->setConfig('mode', $auth_mode);
      return $auth;
      break;
    case "pam":
      $auth = new PAMAuthenticator();
      $auth->setConfig('mode', $auth_mode);
      return $auth;
      break;
    default:
      $auth = new Authenticator();
      $auth->setConfig('mode', "");
      return $auth;
      break;
  }
}


class Authenticator
{
  var $user_id;
  var $username;
  var $retval;
  var $fallback;
  var $AuthConfig = array();
  var $AuthMode;
  
  function Authenticator()
  {
    $this->fallback =  false;
  }


  function authenticate($username, $password)
  {
    return false;
  }

  function userExists($username)
  {
    if ($AuthMode == "") return true;
    return false;
  }

  function userId($username)
  {
    return "";
  }
  
  function setConfig($item, $value)
  {
    $AuthConfig[$item] = $value;
  }
}    

  
class LDAPAuthenticator extends Authenticator
{
  var $ldap_host;
  var $ldap_port;
  var $ldap_version;
  var $base_dn;
  var $ldap_search_user;
  var $ldap_search_pass;	
  var $filter;

  var $user_id;
  var $username;

  function LDAPAuthenticator()
  {
  }

  function authenticate($username, $password)
  {
    $this->fallback = isset($AuthConfig['ldap_allow_login']) ? $AuthConfig['ldap_allow_login'] : false;

    $this->ldap_host = $AuthConfig["ldap_host"];
    $this->ldap_port = $AuthConfig["ldap_port"];
    $this->ldap_version = $AuthConfig["ldap_version"];
    $this->base_dn = $AuthConfig["ldap_base_dn"];
    $this->ldap_search_user = $AuthConfig["ldap_search_user"];
    $this->ldap_search_pass = $AuthConfig["ldap_search_pass"];
    $this->filter = $AuthConfig["ldap_user_filter"];
    
    
    
    $this->username = $username;

    if (strlen($password) == 0) return false; // LDAP will succeed binding with no password on AD (defaults to anon bind)
    if ($this->fallback == true)
    {
      if (parent::authenticate($username, $password)) return true;	
    }

    if (!$rs = @ldap_connect($this->ldap_host, $this->ldap_port))
    {
      return false;
    }
    @ldap_set_option($rs, LDAP_OPT_PROTOCOL_VERSION, $this->ldap_version);
    @ldap_set_option($rs, LDAP_OPT_REFERRALS, 0);

    //$ldap_bind_dn = "cn=".$this->ldap_search_user.",".$this->base_dn;
    $ldap_bind_dn = empty($this->ldap_search_user) ? NULL : $this->ldap_search_user;	
    $ldap_bind_pw = empty($this->ldap_search_pass) ? NULL : $this->ldap_search_pass;

    if (!$bindok = @ldap_bind($rs, $ldap_bind_dn, $this->ldap_search_pass))
    {
      // Uncomment for LDAP debugging
      /*	
      $error_msg = ldap_error($rs);
      die("Couldn't Bind Using ".$ldap_bind_dn."@".$this->ldap_host.":".$this->ldap_port." Because:".$error_msg);
      */
      return false;
    }
    else
    {
      $filter_r = html_entity_decode(str_replace("%USERNAME%", $username, $this->filter), ENT_COMPAT, 'UTF-8');
      $result = @ldap_search($rs, $this->base_dn, $filter_r);
      if (!$result) return false; // ldap search returned nothing or error
      
      $result_user = ldap_get_entries($rs, $result);
      if ($result_user["count"] == 0) return false; // No users match the filter

      $first_user = $result_user[0];
      $ldap_user_dn = $first_user["dn"];

      // Bind with the dn of the user that matched our filter (only one user should match sAMAccountName or uid etc..)

      if (!$bind_user = @ldap_bind($rs, $ldap_user_dn, $password))
      {
        /*
        $error_msg = ldap_error($rs);
        die("Couldnt Bind Using ".$ldap_user_dn."@".$this->ldap_host.":".$this->ldap_port." Because:".$error_msg);
        */
        return false;
      }
      else
      {
        return true;
      } 
    }
  }
}

class PAMAuthenticator extends Authenticator
{
  var $user_id;
  var $username;
  var $retval;

  function PAMAuthenticator()
  {
  }


  function authenticate($username, $password)
  {
    $this->fallback = isset($AuthConfig['pam_allow_login']) ? $AuthConfig['pam_allow_login'] : false;
    
    $this->username = $username;

    if (strlen($password) == 0) return false; // LDAP will succeed binding with no password on AD (defaults to anon bind)
    if ($this->fallback == true)
    {
      if (parent::authenticate($username, $password)) return true;	
    }
    
    exec("echo $password | /home/openvpn/www/status/pam/check_user $username", $thelist, $rc);
    if ($rc == 0) return true;
    return false;
    
    // Bette way would be to use the pam_auth interface, but it doesn't
    // work on all systems.
//    $retval = pam_auth($username, $password);
//    if ($retval == 1)
//    {
//        return true;
//    } 
//    else 
//    {
//      return false;
//    }

  }
}    
  
  
?>
