<?php
if (!defined ("LOCAL_SECURITY_KEY"))
 die ();

// ----------------------------------------------
function openvpn_load_server_conf ()
{
 global $config;
 global $openvpn;
 global $openssl;
 
 global $Server;
 if ($Server)
 {
   $config['openvpn']['config']     = $config['openvpn']['folder'] .$Server . ".conf";
   if (!@file_exists($config['openvpn']['config']))
   {
     $s = strpos($Server, "-");
     if ($s)
     {
       $f = substr($Server, 0, $s);
       $config['openvpn']['config']     = $config['Home'] . "/clients/" .$Server . ".conf";
     } else
     {
       $f = $Server;
       $config['openvpn']['config']     = $config['Home'] . "/servers/" .$Server . ".conf";
     }  
     $config['openvpn']['folder']   = $config['Home'] . "easy-rsa/keys/$f/";
     $config['openssl']['folder']   = $config['Home'] . "easy-rsa/keys/$f/";
   } else
      $config['openssl']['folder']     = $config['Home'] . "easy-rsa/keys/$Server/";
   $config['openvpn']['status']     = $config['openvpn']['folder'] ."openvpn-$Server-status.log";
   
   // Change: openssl keys directory
   $config['openssl']['CA']['priv'] = 'file://'. $config['openssl']['folder'] .$Server.'.key';
   $config['openssl']['CA']['pub']  = 'file://'. $config['openssl']['folder'] .'ca.crt';
   // Change: openssl serial file
   $config['openssl']['serial']     = $config['openssl']['folder'] .'serial';
   // Change: openssl database
   $config['openssl']['database']   = $config['openssl']['folder'] .'index.txt';
   // Change: openssl configuration
   $config['openssl']['config']     = array ('config'      => $config['openvpn']['folder'] .'openssl.cnf',
                                             'encrypt_key' => 0);

 }
 else
 {
   html_error ("No OpenVPN Server specified!");
   exit;
 }
 
 $lines = @file ($config['openvpn']['config']);

 if (!is_array ($lines))
 {
     html_error ("The OpenVPN configuration file: " . $config['openvpn']['config'] . " is empty or does not exist!");
     exit;
 }
 $openvpn['Server']['Name'] = $Server;
 foreach ($lines as $line_num => $line)
 {
  chomp ($line);
  $linetokens = explode (' ', str_strip_spaces ($line));
  switch (strtolower ($linetokens[0]))
  {
   case 'local':          $openvpn['Server']['Local'] =               $linetokens[1]; break;
   case 'proto':          $openvpn['Server']['Proto'] =               $linetokens[1]; break;
   case 'port':           $openvpn['Server']['Port'] =                $linetokens[1]; break;
   case 'dev':            $openvpn['Server']['Dev'] =                 $linetokens[1]; break;
   
   case 'mode':           $openvpn['Server']['Mode'] =                $linetokens[1]; break;
   case 'client':         $openvpn['Server']['Mode'] =                $linetokens[0]; break;
   case 'server':         $openvpn['Server']['Mode'] =                $linetokens[0]; break;
   case 'server-bridge':  $openvpn['Server']['Mode'] =                $linetokens[0]; break;
   
   case 'max-clients':    $openvpn['Server']['MaxClients'] =          $linetokens[1]; break;
   
   case 'fast-io':        $openvpn['Server']['FastIO'] =              $linetokens[0]; break;
   case 'multihome':      $openvpn['Server']['Multihome'] =           $linetokens[0]; break;
   case 'mtu-disc':       $openvpn['Server']['MTUDisc'] =             $linetokens[1]; break;
   case 'mtu-test':       $openvpn['Server']['MTUTest'] =             $linetokens[0]; break;
   case 'mssfix':         $openvpn['Server']['MSSFIX'] =              $linetokens[1];  break;
   case 'fragment':       $openvpn['Server']['Fragment'] =            $linetokens[1];  break;
   case 'management':     
     $openvpn['Server']['Management']['Address'] =  $linetokens[1]; 
     $openvpn['Server']['Management']['Port'] =  $linetokens[2]; 
     break;
   
  case 'status':
    $openvpn['Server']['Status']['Exists'] = if_file_exists ($linetokens[1], true, true);
    $openvpn['Server']['Status']['File']   = $linetokens[1];
    break;
   case 'status-version': $openvpn['Server']['Status']['Version'] =   $linetokens[1]; break;
   case 'ca':
    $openssl['Server']['CA']['Exists'] = if_file_exists ($linetokens[1], true, true);
    $openssl['Server']['CA']['File']   = $linetokens[1];
    // Special code to get the ca.key file here
    $key = basename($linetokens[1], ".crt") . ".key";
    $openssl['Server']['CAkey']['Exists'] = if_file_exists ($key, true, true);
    $openssl['Server']['CAkey']['File']   = $key;
    break;
   case 'dh': 
    $openvpn['Server']['DH']['Exists'] = if_file_exists ($linetokens[1], true, true);
    $openvpn['Server']['DH']['File']   = $linetokens[1];
    break;
   case 'cert':
    $openvpn['Server']['Certificate']['Exists'] = if_file_exists ($linetokens[1], true, true);
    $openvpn['Server']['Certificate']['File']   = $linetokens[1]; 
    break;
   case 'key':            
    $openvpn['Server']['Private_Key']['Exists'] = if_file_exists ($linetokens[1], true, true);
    $openvpn['Server']['Private_Key']['File']   = $linetokens[1];
    break;
   case 'tls-auth':
    $openvpn['Server']['TLS_Auth']['Exists'] = if_file_exists ($linetokens[1], true, true);
    $openvpn['Server']['TLS_Auth']['File']   = $linetokens[1];
    $openvpn['Server']['TLS_Auth']['Key']    = $linetokens[2];
    break;
   case 'crl-verify':     
    $openvpn['Server']['CRL_Verify']['Exists'] = if_file_exists ($linetokens[1], true, true);
    $openvpn['Server']['CRL_Verify']['File']   = $linetokens[1];
    break;
   case 'management':
    $openvpn['Server']['ManagementIP']   = $linetokens[1];
    $openvpn['Server']['ManagementPort']   = $linetokens[2];
    break;
  }
  
  // conf
  // ovpn
  // .sh
  // .zip
  // mac.sh
  if (strpos($Server, "-"))
  {
    $s = $config['Home'] . "/clients/$Server.conf";
  }  else
  {
    $s = $config['Home'] . "/servers/$Server.conf";
  }
  $openvpn['Server']['Conf']['Exists'] = if_file_exists ($s, true, true);
  $openvpn['Server']['Conf']['File']   = $s;
  
  if (strpos($Server, "-"))
  {
    $s = $config['Home'] . "/clients/$Server.ovpn";
  } else
  {
    $s = $config['Home'] . "/servers/$Server.ovpn";
  }
  $openvpn['Server']['Ovpn']['Exists'] = if_file_exists ($s, true, true);
  $openvpn['Server']['Ovpn']['File']   = $s;
  
  if (strpos($Server, "-"))
  {
    $s = $config['Home'] . "/clients/installscripts/openvpn-$Server.zip";
  } else
  {
    $s = $config['Home'] . "/servers/installscripts/openvpn-$Server.zip";
  }
  $openvpn['Server']['zip']['Exists'] = if_file_exists ($s, true);
  $openvpn['Server']['zip']['File']   = $s;
  
  if (strpos($Server, "-"))
  {
    $s = $config['Home'] . "/clients/installscripts/openvpn-$Server.sh";
  } else
  {
    $s = $config['Home'] . "/servers/installscripts/openvpn-$Server.sh";
  }
  $openvpn['Server']['sh']['Exists'] = if_file_exists ($s, true);
  $openvpn['Server']['sh']['File']   = $s;
  
  $s = substr($s, 0, strlen($s) - 3) . ".mac.sh";  
  $openvpn['Server']['mac']['Exists'] = if_file_exists ($s, true);
  $openvpn['Server']['mac']['File']   = $s;
  
  
  
  
  
 }
}

// ----------------------------------------------
function openvpn_load_status ()
{
 global $config;
 global $openvpn;

 $stop_reading = FALSE;
 $openvpn['Server']['Manager'] = FALSE;

 if (!isset ($openvpn['Server']['Status']['File']))
 {
   return;
   html_error ("The OpenVPN status file is not defined!");
   exit;
 }
 
 // try to load status using manager API or fallback to file
 if($lines = openvpn_manager_getstatus()){
  $openvpn['Server']['Manager'] = TRUE;
 }
 else $lines = @file (str_file_fullname ($config['openvpn']['folder'], $openvpn['Server']['Status']['File']));
 $stat = @stat(str_file_fullname ($config['openvpn']['folder'], $openvpn['Server']['Status']['File']));
 
 if ($stat && time() - $stat[9] < 70)
 {
   if (!is_array ($lines))
   {
     $openvpn['Server']['Running'] = -1;
     $openvpn['Server']['Title'] = "";
     return;
   } else
   {
     $openvpn['Server']['Running'] = 1;
     $openvpn['Server']['Seconds_Ago']     = time() - $stat[9]; 
   }
 } else 
 {
   $openvpn['Server']['Running'] = 0;
   return;
 }

 foreach ($lines as $line_num => $line)
 {
  chomp ($line);
  $linetokens = explode (',', $line);

  if ($linetokens[0] == 'TITLE')
   $openvpn['Server']['Title'] = $linetokens[1];
  else if ($linetokens[0] == 'TIME')
  {
   $openvpn['Server']['DateTime'] = $linetokens[1]; 
   $openvpn['Server']['Time']     = $linetokens[2]; 
  }
  else if ($linetokens[0] == 'HEADER' && $linetokens[1] == 'CLIENT_LIST')
  {
   // We will need to know what field is what ?
  }
  else if ($linetokens[0] == 'HEADER' && $linetokens[1] == 'ROUTING_TABLE')
  {
   // We will need to know what field is what ?
  }
  else if ($linetokens[0] == 'CLIENT_LIST')
  {
   $openvpn['Client'][] = array ('CN'          => str_replace ('_', ' ', $linetokens[1]),
                                 'REALADDR'    => $linetokens[2],
                                 'VIRTADDR'    => $linetokens[3],
                                 'BYTESRCV'    => number_format($linetokens[4]),
                                 'BYTESSND'    => number_format($linetokens[5]),
                                 'SINCE'       => $linetokens[6],
                                 'SINCEUT'     => $linetokens[7],
                                 'Seconds_Ago' => seconds_string (time () - $linetokens[7])
                                );
  }
  else if ($linetokens[0] == 'ROUTING_TABLE')
  {
   $openvpn['Routing'][] = array ('VIRTADDR'    => $linetokens[1],
                                  'CN'          => str_replace ('_', ' ', $linetokens[2]),
                                  'REALADDR'    => $linetokens[3],
                                  'LAST'        => $linetokens[4],
                                  'Seconds_Ago' => seconds_string (time () - $linetokens[4])
                                 );
  }
 }
}

// ----------------------------------------------
function openvpn_find_connected ()
{
 global $openvpn;
 global $openssl;

 // find who is connected
 if (isset ($openvpn['Client']))
  $cnconnected = count ($openvpn['Client']);
 else
  $cnconnected = 0;
 $cntotal     = count ($openssl['Database']);

 if ($cnconnected > 0)
  for ($i = 0; $i < $cnconnected; $i++)
   for ($j = 0; $j < $cntotal; $j++)
    if ($openvpn['Client'][$i]['CN'] == $openssl['Database'][$j]['CN'])
    {
     $openssl['Database'][$j]['Connected'] = true;
     break;
    } 
}


function openvpn_manager_connect () {
 global $config;
 global $openvpn;
 
 if ($openvpn['Server']['Management']['Address'] != "")
     $config['openvpn']['manager']['address'] = $openvpn['Server']['Management']['Address'];
 if ($openvpn['Server']['Management']['Port'] != "")
     $config['openvpn']['manager']['port'] = $openvpn['Server']['Management']['Port'];
 if(!isset($config['openvpn']['manager']['address']) || !isset($config['openvpn']['manager']['port']))
  return false;

 ini_set("default_socket_timeout",5);
 //$uri='tcp://'.$config['openvpn']['manager']['address'].':'.(int)$config['openvpn']['manager']['port'];
 $uri='tcp://'.$config['openvpn']['manager']['address'];
 $fp=@fsockopen($uri, $config['openvpn']['manager']['port']);
 if (!$fp)
  return false;

 // OpenVPN status line
 $status=fgets($fp, 512);
 if(!preg_match('/OpenVPN/',$status)) {
  if($fp) fclose($fp);
  return false;
 }
 return $fp;
}

function openvpn_manager_disconnect ($fp) {
 // send quit to terminate telnet session
 if($fp){
  fwrite($fp, "quit\n");
  fclose($fp);
 }
 return;
}

function openvpn_manager_sendcommand ($command)
{
 $fp=openvpn_manager_connect();
 if(!$fp) return false;
        
 $st=$command."\n";
 if (fwrite($fp, $st)!=strlen($st))
 {
    if($fp) fclose($fp);
    return false;
}
 // get response
 $response=fgets($fp, 512);
 openvpn_manager_disconnect($fp);
 return $response;
}

function openvpn_manager_getstatus ()
{
  global $openvpn;

 $fp=openvpn_manager_connect();
 if(!$fp) return false;
 fwrite($fp,"status 2\n");
 // get response
 $buffer=array();
 while (!feof($fp)) {
  $str = trim(fgets($fp, 4096));
  $buffer[]=$str;
  if($str=='END' || preg_match('/^ERROR:/',$str)) break;
 }
 openvpn_manager_disconnect($fp);
 return $buffer;
}


?>
