/*
  http://www.kernel.org/pub/linux/libs/pam/Linux-PAM-html/adg-example.html
  http://www.cs.utah.edu/dept/old/texinfo/glibc-manual-0.02/library_15.html

  This program was contributed by Shane Watts
  [modifications by AGM and kukuk]

  You need to add the following (or equivalent) to the
  /etc/pam.d/check_user file:
  # check authorization
  auth       required     pam_unix.so
  account    required     pam_unix.so
 */

#include <security/pam_appl.h>
#include <security/pam_misc.h>
#include <stdio.h>


typedef struct {
        char *name, *pw;
} pam_auth_t;



/*
 * auth_pam_talker: supply authentication information to PAM when asked
 *
 * Assumptions:
 *   A password is asked for by requesting input without echoing
 *   A username is asked for by requesting input _with_ echoing
 *
 */
static
int auth_pam_talker(int num_msg,
                                const struct pam_message ** msg,
                                struct pam_response ** resp,
                                void *appdata_ptr)
{
        unsigned short i = 0;
        pam_auth_t *userinfo = (pam_auth_t *) appdata_ptr;
        struct pam_response *response = 0;
        /* parameter sanity checking */
        if (!resp || !msg || !userinfo)
                return PAM_CONV_ERR;

        /* allocate memory to store response */
        response = malloc(num_msg * sizeof(struct pam_response));
        if (!response)
                return PAM_CONV_ERR;
        /* copy values */
        for (i = 0; i < num_msg; i++) {
                /* initialize to safe values */
                response[i].resp_retcode = 0;
                response[i].resp = 0;

                /* select response based on requested output style */
                switch (msg[i]->msg_style) {
                        case PAM_PROMPT_ECHO_ON:
                                /* on memory allocation failure, auth fails */
                                response[i].resp = strdup(userinfo->name);
                                break;
                        case PAM_PROMPT_ECHO_OFF:
                                response[i].resp = strdup(userinfo->pw);
                                break;
                        default:
                                if (response)
                                    free(response);
                                return PAM_CONV_ERR;
                }
        }
        /* everything okay, set PAM response values */
        *resp = response;
        return PAM_SUCCESS;
}




int main(int argc, char *argv[])
{
    pam_handle_t *pamh=NULL;
    int  result;
    const char *user="nobody";
    char password[128];
    
    pam_auth_t userinfo = {NULL, NULL};
    struct pam_conv conv_info = {&auth_pam_talker, (void *) &userinfo};

        
    if(argc == 2) {
        user = argv[1];
    }

    if(argc > 2) 
    {
        fprintf(stderr, "Usage: check_user [username]\n");
        exit(1);
    }
    
    
    fgets(password, sizeof(password), stdin);
    if (password[strlen(password) - 1] == '\n') {
        password[strlen(password) - 1] == '\0';
    }
    
    
    userinfo.name = argv[1];
    userinfo.pw = password;
    
    
    if ((result = pam_start("check_user", userinfo.name, &conv_info, &pamh)) != PAM_SUCCESS) {
        return 0;
    }
    if ((result = pam_authenticate(pamh, PAM_DISALLOW_NULL_AUTHTOK)) != PAM_SUCCESS) {
        return 0;
    }
    
    /* This is where we have been authorized or not. */

    if (result == PAM_SUCCESS) {
        fprintf(stdout, "Authenticated\n");
    } else {
        fprintf(stdout, "Not Authenticated\n");
    }

    if (pam_end(pamh,result) != PAM_SUCCESS) {     /* close Linux-PAM */
        pamh = NULL;
        fprintf(stderr, "check_user: failed to release authenticator\n");
        exit(1);
    }

    return ( result == PAM_SUCCESS ? 0:1 );       /* indicate success */
}

